@@ -1,5 +1,5 @@
 /*-*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: Library.java,v 1.4 2000/02/11 22:08:18 metlov Exp $
+ * $Id: Library.java,v 1.5 2000/08/15 21:37:55 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -51,7 +51,22 @@ public class Library {
   private Hashtable dynIDs; // Hashtable(members, integers)
 
   private Hashtable stateless; // Hashtable(members, booleans)
+
+  private Hashtable dotClasses; // Hashtable(classes, Hashtable(names, Hashtable(signatures, members)))
+  private boolean noDotSecurity=false;
   
+  /**
+   * Creates a library for JEL.
+   * <P> See the three argument constructor for more info.
+   * @param staticLib is the array of classes, whose public static 
+   *  methods are exported.
+   * @param dynamicLib is the array of classes, whose public virutal
+   *  methods are exported.
+   */
+  public Library(Class[] staticLib,Class[] dynamicLib) {
+    this(staticLib,dynamicLib,null);
+  };
+
   /**
    * Creates a library for JEL.
    * <P> The following should be kept in mind when constructing a library:
@@ -77,22 +92,61 @@ public class Library {
    * debugging is ON (see <TT>gnu.jel.debug.Debug.enabled</TT>). This is
    * done to avoid unnecessary error messages in the production code of the
    * compiler.
+   * <P>The array (dotClasses), which is the third argument of 
+   * this constructor determines how (and whether) to compile the 
+   * dot operators encountered in expressions. These operators are
+   * of the form <object>.(<method>|<field>), 
+   * which means to call method (access field)
+   * of an <object>. There can be three types of the behaviour: 
+   * <BR>1) dot operators are prohibited (<TT>dotClasses==null</TT>),
+   * this is behaviour of older version of JEL.
+   * <BR>2) dot operators are allowed on all classes
+   * (<TT>dotClasses==new Class[0], an empty array).
+   * Depending on the types of objects returned by the static/dynamic library
+   * classes this may pose a security risk.
+   * <BR>3) dot operators are allowed only on some classes. This is achieved 
+   * by listing these classes in the dotClasses array.
    * @param staticLib is the array of classes, whose public static 
    *  methods are exported.
    * @param dynamicLib is the array of classes, whose public virutal
    *  methods are exported.
+   * @param dotClasses is the array of classes on which the dot ('.')
+   * operation is allowed.
    */
-  public Library(Class[] staticLib,Class[] dynamicLib) {
+  public Library(Class[] staticLib,Class[] dynamicLib,Class[] dotClasses) {
+    if (dotClasses==null) {
+      this.dotClasses=null;
+    } else {
+      noDotSecurity=(dotClasses.length==0);
+      this.dotClasses=new Hashtable();
+      // hash the names
+      Class[] temp=new Class[1];
+      for(int i=0;i<dotClasses.length;i++) 
+        rehashDotClass(dotClasses[i]);
+    };
+
     names = new Hashtable();
     dynIDs = new Hashtable();
     stateless=new Hashtable();
+
     if (staticLib!=null)
-      rehash(staticLib,true);
+      rehash(staticLib,names,null,stateless);
     if (dynamicLib!=null)
-      rehash(dynamicLib,false);
+      rehash(dynamicLib,names,dynIDs,null);
+
   };
-  
-  private void rehash(Class[] arr, boolean gather_stateless) {
+
+  private void rehashDotClass(Class cls) {
+    Hashtable tempNames=new Hashtable();
+    Class[] temp=new Class[1];
+    temp[0]=cls;
+    //    rehash(temp,tempNames,null,new Hashtable());
+    rehash(temp,tempNames,new Hashtable(),null);
+    dotClasses.put(cls,tempNames);
+  };
+
+  private static void rehash(Class[] arr, Hashtable hashedNames, 
+                             Hashtable dynIDs, Hashtable stateless) {
     for (int i=0; i<arr.length; i++) {
       Integer dynID=new Integer(i);
 
@@ -106,15 +160,17 @@ public class Library {
           (Member)new Method(marr[j]):
           (Member)new Field(farr[j-totalMethods]);
         if ((m.getModifiers() & 0x0008)>0) { // static
-          if (gather_stateless && rehash(m)) stateless.put(m,Boolean.TRUE);
+          if ((stateless!=null) && rehash(hashedNames,m)) 
+            stateless.put(m,Boolean.TRUE);
         } else { // not static
-          if (!gather_stateless && rehash(m)) dynIDs.put(m,dynID);
+          if ((dynIDs!=null) && rehash(hashedNames,m))
+            dynIDs.put(m,dynID);
         };
       };
     };
   };
 
-  private boolean rehash(Member m) {
+  private static boolean rehash(Hashtable hashedNames, Member m) {
     String name=m.getName();
     String signature=ClassFile.getSignature(m);
 
@@ -122,12 +178,12 @@ public class Library {
     // arguments
     if (m instanceof Field) signature="()"+signature;
     
-    Hashtable signatures=(Hashtable)names.get(name);
+    Hashtable signatures=(Hashtable)hashedNames.get(name);
     if (signatures==null) { 
       // No method with this name was added
       Hashtable signatures_new=new Hashtable();
       signatures_new.put(signature,m);
-      names.put(name,signatures_new);
+      hashedNames.put(name,signatures_new);
       return true;
     };
     // Name exists in the library, check for possible signature conflict.
@@ -199,10 +255,52 @@ public class Library {
    */
   public Member getMember(String name,Class[] params) 
     throws NoSuchMethodException {
-    Hashtable signatures=(Hashtable) names.get(name);
+    return getMember(names,null,name,params);
+  };
+
+  /**
+   * Finds the member of a given class by name.
+   * <P> This method is used in compiling the dot ('.') operators.
+   * It does all necessary security checks and wraps the result
+   * into one of JEL's reflection classes.
+   * @param container the class to search the method within.
+   * @param name is the name of the method to find.
+   * @param params are the types of formal parameters in the method invocation.
+   * @return the method/field object of the resolved method/field.
+   * @exception NoSuchMethodException if the method can't be resolved
+   */
+  public Member getMember(Class container,String name,Class[] params) 
+    throws NoSuchMethodException {
+    // security
+    if (dotClasses==null)
+      throw new NoSuchMethodException("The access to class members is "+
+                                      "not allowed.");
+    else if (! (noDotSecurity || dotClasses.containsKey(container)))
+      throw new NoSuchMethodException("The access to members of "+
+                                      container.toString()+
+                                      " is not allowed.");
+    Hashtable hashedMembers=(Hashtable)dotClasses.get(container);
+    if (hashedMembers==null) {
+      rehashDotClass(container);
+      hashedMembers=(Hashtable)dotClasses.get(container);
+    };
+      
+    return getMember(hashedMembers,container,name,params);;
+  };
+
+
+  // This matches a method to a given invocation pattern,
+  // unlike Class.getMethod(...) tries various widening conversions.
+  private static Member getMember(Hashtable hashedMembers, Class cls,
+                                  String name,Class[] params) 
+    throws NoSuchMethodException {
+    Hashtable signatures=(Hashtable) hashedMembers.get(name);
 
     if (signatures==null)
-      throw new NoSuchMethodException("The name \""+name+"\" is not defined.");
+      throw new NoSuchMethodException("The name \""+name+
+                                      "\" is not defined "+
+                                      (cls==null?"":"in the "+cls)+
+                                      " .");
     
     // Choose applicable methods
     Vector applicable_methods=new Vector();
@@ -226,10 +324,11 @@ public class Library {
     };
     
     if (applicable_methods.size()==0) {
-      throw new NoSuchMethodException("Function \""+name+"\" exists,"+
-				      " but parameters "+
-				      describe(name,params)+
-				      " can not be accepted by it.");
+      throw new NoSuchMethodException("Function \""+name+"\" exists"+
+                                      (cls==null?"":" in the "+cls)+
+                                      ", but parameters "+
+                                      describe(name,params)+
+                                      " can not be accepted by it.");
     };
     
     if (applicable_methods.size()==1) 
@@ -262,12 +361,13 @@ public class Library {
                                         describe(name,most_specific_params) +
                                         "\" and \""+ describe(name,cp) +
                                         "\" on invocation \""+
-                                        describe(name,params)+"\" .");
+                                        describe(name,params)+"\""+
+                                        (cls==null?"":" on the "+cls)+" .");
     };
     return most_specific;
   };
 
-  private String describe(String name,Class[] params) {
+  private static String describe(String name,Class[] params) {
     String res="";
     StringBuffer invp=new StringBuffer();
     invp.append(name); 
@@ -474,6 +574,63 @@ public class Library {
       t.testFail();
     };
 
+    t.startTest("Creating a library with dot operations allowed ");
+    Library ldot=null;
+    try {
+      Class[] stl=new Class[1];
+      Class[] dynl=new Class[1];
+      Class[] dotl=new Class[2];
+      stl[0]=Class.forName("java.lang.Math");
+      dynl[0]=Class.forName("java.util.Random");
+      dotl[0]=Class.forName("java.util.Hashtable");
+      dotl[1]=Class.forName("java.util.Vector");
+      
+      ldot=new Library(stl,dynl,dotl);
+      t.testOK();
+    } catch (Throwable e) {
+      Debug.reportThrowable(e);
+      t.testFail();
+    };
+
+    t.startTest("Resolving size() on Hashtable");
+    try {
+      Class[] params=new Class[0];
+      Class htable=Class.forName("java.util.Hashtable");
+      Member mf=ldot.getMember(htable,"size",params);
+      if ((mf!=null) && 
+          (mf.equals(new Method(htable.getMethod("size",params)))))
+	  t.testOK(); else t.testFail();
+    } catch (Throwable e) {
+      Debug.reportThrowable(e);
+      t.testFail();
+    };
+
+    t.startTest("Resolving size() on Vector");
+    try {
+      Class[] params=new Class[0];
+      Class vctr=Class.forName("java.util.Vector");
+      Member mf=ldot.getMember(vctr,"size",params);
+      if ((mf!=null) && 
+          (mf.equals(new Method(vctr.getMethod("size",params)))))
+	  t.testOK(); else t.testFail();
+    } catch (Throwable e) {
+      Debug.reportThrowable(e);
+      t.testFail();
+    };
+
+    t.startTest("Ensuring size() is not leaked to global context.");
+    try {
+      Class[] params=new Class[0];
+      try {
+        Member mf=ldot.getMember("size",params);
+        t.testFail();
+      } catch (NoSuchMethodException exc) {
+        t.testOK();
+      };
+    } catch (Throwable e) {
+      Debug.reportThrowable(e);
+      t.testFail();
+    };
 
   };
 
