@@ -1,5 +1,5 @@
 /* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * $Id: TestSuite.java,v 1.15 2000/02/12 17:31:32 metlov Exp $
+ * $Id: TestSuite.java,v 1.16 2000/08/15 21:37:55 metlov Exp $
  *
  * This file is part of the Java Expressions Library (JEL).
  *   For more information about JEL visit :
@@ -291,11 +291,31 @@ public class TestSuite {
     
     testExpression("(1+6)/(2+2-4)",null,null,rtp,lib,o);
     testExpression("throw_arg_eq_4(6-2)",null,null,rtp,lib,o);
+
+    o.println(); o.println("----- DOT OPERATOR ------");
+    // Construct a new library
+    Class[] dotLib=new Class[1];
+    try {
+      dotLib[0]=Class.forName("java.lang.String");
+    } catch(ClassNotFoundException e) {
+      System.out.println(e);
+      System.exit(1);
+    };
+    
+    lib=new Library(staticLib,dynamicLib,dotLib);
+
+    testExpression("(\"abc\").length()", new Integer(3), null, rtp, lib,o);
+    testExpression("\"abc\".length()", new Integer(3), null, rtp, lib,o);
+    testExpression("\"abc\".endsWith(\"bc\")", Boolean.TRUE, null, rtp, lib,o);
+    testExpression("\"abcdbc\".indexOf(\"bc\")", new Integer(1), null, rtp, lib,o);
+    testExpression("\"abcdbc\".indexOf(\"abc\".substring(1),2)", new Integer(4), null, rtp, lib,o);
+
+    testError("\"abc\".nomethod()",null,lib,7,o);
+    testError("\"abc\".length(666)",null,lib,7,o);
     
     o.println(); o.println("^^^^^^^^^^^^ SCORE success/failure = "+
                            successes+"/"+failures+
                            " ^^^^^^^^^^^^" );
-
   };
 
   // Tests evaluation of logical expressions
